#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "kk_data_mng.h"
#include "kk_lan_node_db.h"

#include "kk_log.h"

#include "klist.h"
#include "kk_product.h"
#include "com_api.h"
#include "kk_opcode.h"
#include "kk_lan_ctrl.h"
#include "kk_data_mng.h"

#include "kk_oldccu_msg.h"



cJSON *old_ccu_msg_build_json_node_int(int nodeid,const char*  opcode,const char*  status,cJSON * arg)
{
	cJSON *root;

	if(nodeid==NULL||opcode==NULL||arg==NULL) {
		return NULL;
	}
	
	root=cJSON_CreateObject();
	if(root){
		cJSON_AddNumberToObject(root, "nodeid", nodeid);
		cJSON_AddStringToObject(root, "opcode", opcode);
		cJSON_AddStringToObject(root, "status", "success");
		cJSON_AddItemToObject(root, "arg", arg);
	}
	return root;
}


cJSON *old_ccu_msg_build_json(const char* nodeid,const char*  opcode,const char*  status,cJSON * arg)
{
	cJSON *root;

	if(nodeid==NULL||opcode==NULL||arg==NULL) {
		return NULL;
	}
	
	root=cJSON_CreateObject();
	if(root){
		cJSON_AddStringToObject(root, "nodeid", nodeid);
		cJSON_AddStringToObject(root, "opcode", opcode);
		cJSON_AddStringToObject(root, "status", "success");
		cJSON_AddItemToObject(root, "arg", arg);
	}
	return root;
}

cJSON *old_ccu_msg_build(const char* nodeid,const char* opcode,const char* status,const char* arg)
{
	cJSON *root;

	if(nodeid==NULL||opcode==NULL||arg==NULL) {
		return NULL;
	}
	
	root=cJSON_CreateObject();
	if(root){
		cJSON_AddStringToObject(root, "nodeid", nodeid);
		cJSON_AddStringToObject(root, "opcode", opcode);
		if(status==NULL){
			cJSON_AddStringToObject(root, "status", "success");
		}else{
			cJSON_AddStringToObject(root, "status", status);
		}
		cJSON_AddStringToObject(root, "arg", arg);
	}
	return root;
}

int send_msg_to_module(cJSON *root)
{
	char *msg;
	
	if(root==NULL){
		return -1;
	}

	if((msg = cJSON_Print(root))!=NULL){
		printf("[lan->module]json:\n%s\n",msg);
		cJSON_Minify(msg);
		kk_send_data_to_sdk(msg);
		free(msg);
	}

	return 0;
}








cJSON * val_conver_new2old(cJSON *newccuItem,cJSON *oldccuItem,int syn_type)
{
	cJSON *n_dataType = cJSON_GetObjectItem(newccuItem,DATATYPE_STRING);
	cJSON * n_value  = cJSON_GetObjectItem(newccuItem,"value");

	cJSON *o_dataType;
	cJSON *val;


	cJSON *o_valueRange = cJSON_GetObjectItem(oldccuItem,"valueRange");
	cJSON *n_valueRange = cJSON_GetObjectItem(newccuItem,"valueRange");

	if(syn_type==1){
		o_dataType = cJSON_GetObjectItem(oldccuItem,"synType");
	}else{
		o_dataType = cJSON_GetObjectItem(oldccuItem,DATATYPE_STRING);
	}
	
	if(strcmp(o_dataType->valuestring,"map")==0){
		val = map_type_convert(n_dataType,n_valueRange,n_value,o_valueRange);
	}else{
		val = msg_convert_value(o_dataType,n_dataType,n_value);
	}
	return val;
}


typedef struct {
	char *area;
	char *name;
	char *delay;
	char *nodeid;
	char *operate_type;
	char *operation;
	char *room_id;
}SCENE_ACTION;

typedef struct{
	char *scene_id;
	char *name;
	char *pannel_id;
	char *room_id;
	char *time;
	char *week;
	SCENE_ACTION **act;
	int act_num;
}SCENE_BUILD;



cJSON *kk_action_build(SCENE_ACTION *act)
{
	cJSON *action = cJSON_CreateObject();
	if(act->nodeid==NULL || act->operation==NULL ||
		act->operation==NULL ||act->room_id==NULL){
		return ;
	}
	cJSON_AddStringToObject(action, "area",(act->area==NULL)?"":act->area);
	cJSON_AddStringToObject(action, "name",(act->name==NULL)?"":act->name);
	cJSON_AddStringToObject(action, "delay",(act->delay==NULL)?"0":act->delay);
	cJSON_AddStringToObject(action, "nodeid",act->nodeid);
	cJSON_AddStringToObject(action, "operate_type",act->operate_type);
	cJSON_AddStringToObject(action, "operation",act->operation);
	cJSON_AddStringToObject(action, "room_id",act->room_id);
	return action;
}


cJSON *kk_actions_build(SCENE_ACTION **act,int num)
{
	int i;
	cJSON *actions = cJSON_CreateArray();
	cJSON *action;
	for(i=0;i<num;i++){
		action=kk_action_build(act[i]);
		cJSON_AddItemToArray(actions,action);
	}
	return actions;
}

cJSON *kk_scene_build(SCENE_BUILD *content)
{
	cJSON *scene = cJSON_CreateObject();
	cJSON *actions = kk_actions_build(content->act,content->act_num);
	
	cJSON_AddItemToObject(scene, "actions",actions);
	cJSON_AddStringToObject(scene, "id",content->scene_id);
	cJSON_AddStringToObject(scene, "name",content->name);
	cJSON_AddStringToObject(scene, "pannel_id",(content->pannel_id==NULL)?"*":content->pannel_id);
	cJSON_AddStringToObject(scene, "room_id",content->room_id);
	cJSON_AddStringToObject(scene, "scene_image","");
	cJSON_AddStringToObject(scene, "scene_type","");
	if(content->time==NULL){
		cJSON_AddStringToObject(scene, "timer_enable","0");
		cJSON_AddStringToObject(scene, "time","");
	}else{
		cJSON_AddStringToObject(scene, "timer_enable","1");
		cJSON_AddStringToObject(scene, "time",content->time);
	}
	
	cJSON_AddStringToObject(scene, "week",content->week);
	
	return scene;
	
}

void kk_scene_build_test(void)
{
	cJSON *msg;
	SCENE_BUILD scene;
	scene.name = calloc(1,20);
	memcpy(scene.name,"离家",strlen("离家"));
	scene.scene_id = calloc(1,20);
	memcpy(scene.scene_id,"123",strlen("123"));

	scene.pannel_id= calloc(1,20);
	memcpy(scene.pannel_id,"*",strlen("*"));
	scene.room_id = calloc(1,20);
	memcpy(scene.room_id,"456",strlen("456"));
	scene.time = calloc(1,20);
	memcpy(scene.time,"",strlen(""));
	scene.week = calloc(1,20);
	memcpy(scene.week,"",strlen(""));


	scene.act_num = 1;
	
	SCENE_ACTION *act = calloc(1,sizeof(SCENE_ACTION)*scene.act_num);
	scene.act=&act;
	scene.act[0]->area=calloc(1,20);
	memcpy(scene.act[0]->area,"",strlen(""));

	scene.act[0]->delay=calloc(1,20);
	memcpy(scene.act[0]->delay,"",strlen(""));
	
	scene.act[0]->name=calloc(1,20);
	memcpy(scene.act[0]->name,"",strlen(""));

	scene.act[0]->nodeid=calloc(1,20);
	memcpy(scene.act[0]->nodeid,"",strlen(""));

	scene.act[0]->operate_type=calloc(1,20);
	memcpy(scene.act[0]->operate_type,"",strlen(""));

	scene.act[0]->operation=calloc(1,20);
	memcpy(scene.act[0]->operation,"",strlen(""));

	scene.act[0]->room_id=calloc(1,20);
	memcpy(scene.act[0]->room_id,"",strlen(""));

	msg = kk_scene_build(&scene);

	printf("--------scene--------->\n%s\n",cJSON_Print(msg));

}



cJSON *kk_devicestatus_build(kk_map_dev_node_t *node)
{
	char nodeid[32] = {0};
	char indxId[4] = {0};
	int idx = 1;
	int syn_type = 0;

	cJSON *dev_status;

	if (node == NULL) return NULL;

	dev_status = cJSON_CreateObject();

	kk_map_dev_ctx *ctx = _kk_map_dev_ctx();

	#if 1

	cJSON * newccuItem = node->newccu->child;
	cJSON * oldccuItem = node->oldccu->child;

	syn_type = node->syn_type;

	{
		cJSON *status = cJSON_CreateObject();

		cJSON *synKey;
		cJSON *synType;
	

		printf("yyy-1\n");
		memset(nodeid,0x0,sizeof(nodeid));
			printf("yyy-2\n");
		kk_creater_nodeid(node->deviceCode,1,nodeid);
		printf("yyy-3,%s\n",nodeid);

		if(node->syn_opcode){
			cJSON_AddStringToObject(status, OPCODE_STRING, node->syn_opcode);
		}
		printf("yyy-4\n");
		cJSON_AddNumberToObject(status, NODEID_STRING, atoi(nodeid));
		printf("yyy-5\n");
		sprintf(indxId,"%d",idx++);
		printf("yyy-6\n");
		cJSON_AddStringToObject(status, "index", indxId);
		printf("yyy-7\n");

		while(newccuItem != NULL){
		
			//printf("oldccuItem--->%s\n",cJSON_Print(oldccuItem));

			synType = cJSON_GetObjectItem(oldccuItem,"synType");

			printf("synType->valuestring--->%s\n",synType->valuestring);
		
			if(strcmp(synType->valuestring,"dummy")){
				synKey = cJSON_GetObjectItem(oldccuItem,"syn");

				if(strcmp(synType->valuestring,"timing_shutdown")==0) {

					cJSON *syn_obj = cJSON_CreateObject();

					double t=cJSON_GetObjectItem(newccuItem,"value")->valuedouble;
					char buff[32]={0};

					snprintf(buff,32,"%f",t);
					printf("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&%s\n",buff);
						
					if(t>0){
						cJSON_AddBoolToObject(syn_obj, "enable",1);
						cJSON_AddStringToObject(syn_obj, "time",buff);
					}else{
						cJSON_AddBoolToObject(syn_obj, "enable",0);
						cJSON_AddNullToObject(syn_obj, "time");
					}

						cJSON_AddItemToObject(dev_status, synKey->valuestring,syn_obj);
					
				}else{
					cJSON *val = val_conver_new2old(newccuItem,oldccuItem,1);

					

					if(val){
						printf("val--->%s\n",cJSON_Print(val));
						printf("val->type=%d,%d,%d\n",val->type,cJSON_String,cJSON_Number);
						if(val->type==cJSON_String) {
							cJSON_AddStringToObject(dev_status, synKey->valuestring,val->valuestring);
						}else if(val->type==cJSON_Number) {
							cJSON_AddNumberToObject(dev_status, synKey->valuestring,val->valuedouble);
						}else{
							cJSON_AddItemToObject(dev_status, synKey->valuestring,val);
						}
					}

				}
			}

			newccuItem = newccuItem->next;
			oldccuItem = oldccuItem->next;
			
		}
	}
		

	 
	#endif

	printf("@##$%^&&*(*(((*^%$$#@@@--->%s\n",cJSON_Print(dev_status));

	return dev_status;
}


int kk_devicestatus_build_aaa(cJSON *Array,kk_map_dev_node_t *node)
{
	char nodeid[32] = {0};
	char indxId[4] = {0};
	int idx = 1;


	cJSON *dev_status;

	if (node == NULL) return -1;

	

	kk_map_dev_ctx *ctx = _kk_map_dev_ctx();


	cJSON * newccuItem = node->newccu->child;
	cJSON * oldccuItem = node->oldccu->child;

	
	while(newccuItem != NULL){
		dev_status = cJSON_CreateObject();

		memset(nodeid,0x0,sizeof(nodeid));

		char *opcodemap = cJSON_GetObjectItem(newccuItem,OPCODEMAP_STRING)->valuestring;
		char *channel = cJSON_GetObjectItem(newccuItem,CHANNEL_STRING)->valuestring;
		char *dataType = cJSON_GetObjectItem(newccuItem,DATATYPE_STRING)->valuestring;
		cJSON *range = cJSON_GetObjectItem(newccuItem,VALUERANGE_STRING);

		char *o_dataType = cJSON_GetObjectItem(oldccuItem,DATATYPE_STRING)->valuestring;
		
		printf("identifier=%s\n",cJSON_GetObjectItem(newccuItem,"identifier")->valuestring);
	
		printf("newccuItem=%s\n",cJSON_Print(newccuItem));

			
		kk_creater_nodeid(node->deviceCode,atoi(channel),nodeid);

		cJSON_AddStringToObject(dev_status, OPCODE_STRING, opcodemap);
		cJSON_AddNumberToObject(dev_status, NODEID_STRING, atoi(nodeid));
		
		sprintf(indxId,"%d",idx++);
		cJSON_AddStringToObject(dev_status, "index", indxId);
		printf("index=%s,dataType=%s,o_dataType=%s\n",indxId,dataType,o_dataType);


		if(strcmp(o_dataType,"string") == 0){
			if(strcmp(dataType,"int") == 0 || strcmp(dataType,"double") == 0){
				int value  = cJSON_GetObjectItem(newccuItem,"value")->valueint;
				if(range != NULL){
					int index = kk_get_int_value_idx(range,value);
					if(index==-1){
						index = 0;
					}
					cJSON * ary=cJSON_GetObjectItem(oldccuItem,"valueRange");
					cJSON * def=cJSON_GetArrayItem(ary,index);
					cJSON_AddStringToObject(dev_status, "arg",def->valuestring);
					printf("yjq=%d\n",index);
					
				}
			}else if(strcmp(dataType,"double") == 0){
				
			}

		}else if(strcmp(o_dataType,"string_double") == 0){

			double value = cJSON_GetObjectItem(newccuItem,"value")->valuedouble;
			char *val = double_value_string(value);
			cJSON_AddNumberToObject(dev_status, "arg",value);
			free(val);
		}

		cJSON_AddItemToArray(Array,dev_status);

		
		//todo 
		newccuItem = newccuItem->next;
		oldccuItem = oldccuItem->next;
		

	}
	

	return 0;
}






cJSON * attr_report_conver(const char *deviceCode,kk_map_dev_node_t *devNode,cJSON *params)
{
	cJSON *root;
	//
	cJSON *newccu;
	cJSON *n_item;
	cJSON *n_id,*n_valueRange,*n_dataType;
	cJSON *n_value;


	//
	cJSON *oldccu;
	cJSON *o_item;
	cJSON *o_id_map,*o_valueRange,*o_dataType;
	cJSON *o_value;


	cJSON *val;
	cJSON *item_size;
	cJSON *channel;
	cJSON *opcode;
	

	int i,j;
	int ch;
	int nodeId;
	char *node_str;
	cJSON *msg;
	cJSON *args;



	
	
	oldccu = devNode->oldccu;
	newccu = devNode->newccu;

	item_size = cJSON_GetArraySize(oldccu);
	
	printf("item_size=%d\n",item_size);
	

	
	for(i=0;i<item_size;i++){
		

		o_item = cJSON_GetArrayItem(oldccu,i);
		n_item = cJSON_GetArrayItem(newccu,i);
	
		o_id_map = cJSON_GetObjectItem(o_item,"identifiermap");
		printf("o_id_map=%s\n",o_id_map->valuestring);

	
		if(o_id_map){
			printf("o_id_map=%s\n",cJSON_Print(o_id_map));
		}
		n_id = cJSON_GetObjectItem(params,o_id_map->valuestring);
		printf("[%s][%d]n_id=%s\n",__FUNCTION__,__LINE__,cJSON_Print(n_id));

		if(n_id!=NULL){
			if(params){
				printf("21-params=%s\n",cJSON_Print(params));
			}
			n_dataType = cJSON_GetObjectItem(n_item,"dataType");
			n_valueRange = cJSON_GetObjectItem(n_item,"valueRange");
		
			o_dataType = cJSON_GetObjectItem(o_item,"dataType");
			o_valueRange = cJSON_GetObjectItem(o_item,"valueRange");
			


			printf("[%s][%d]\n",__FUNCTION__,__LINE__);
			channel = cJSON_GetObjectItem(o_item,"channel");
			ch = atoi(channel->valuestring);
			nodeId = kk_lan_db_node_get(deviceCode,ch);
			
			opcode =cJSON_GetObjectItem(o_item,"opcode");
			
			printf("[%s][%d]n_dataType->valuestring=%s,opcode->valuestring=%s\n",__FUNCTION__,__LINE__,n_dataType->valuestring,opcode->valuestring);

			
			printf("\nn_id->type=%d\n",n_id->type);
			//cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateString(""));//
			if(n_id->type==cJSON_False){
				cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateFalse());
			}else if(n_id->type==cJSON_True){
				cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateTrue());
			}else if(n_id->type==cJSON_NULL){
				cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateNull());
			}else if(n_id->type==cJSON_Number){
				if(strcmp(n_dataType->valuestring,"double")==0){
					cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateNumber(n_id->valuedouble));
				}else {
					cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateNumber(n_id->valueint));
				}
				
			}else if(n_id->type==cJSON_String){
				cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateString(n_id->valuestring));
			}else if(n_id->type==cJSON_Array){
				//cJSON_ReplaceItemInObject(n_item, "value", cJSON_CreateArray(n_id->valuestring));
				/*
				#define cJSON_Array  (1 << 5)
				#define cJSON_Object (1 << 6)
				#define cJSON_Raw    (1 << 7) 
				*/
				printf("......................\n");
			}
			
		
			//cJSON_ReplaceItemInObject(newccuItem, "value", cJSON_CreateNumber(val));


			
			printf("o_dataType=%s\n",o_dataType->valuestring);

			if(strcmp(o_dataType->valuestring,"map")==0){
				args = map_type_convert(n_dataType,n_valueRange,n_id,o_valueRange);
			}else{
				args = msg_convert_value(o_dataType,n_dataType,n_id);
			}
			
			if(args){
				printf("yy111y,args=%s\n",cJSON_Print(args));
				update_device_status_2_oldccu(nodeId,opcode->valuestring,args->valuestring);
			}else{
				printf("yy111y,args=NULL\n");
			}
			

	
			
		}
	}
	printf("yy2222y\n");

	if(devNode->syn_type==1){
		cJSON *arg;
		char *node_str;

		arg = kk_devicestatus_build(devNode);
		printf("yy444y\n");
		nodeId = kk_lan_db_node_get(deviceCode,1);
		printf("yy555y\n");
		node_str=node_string(nodeId);
		printf("yy666y\n");

	
		msg=old_ccu_msg_build_json(node_str,devNode->syn_opcode,NULL,arg);
		printf("66666666666=%s\n",cJSON_Print(msg));
		send_msg_to_module(msg);
		free(node_str);
		cJSON_Delete(msg);
	}	
	
	return root;
}





int del_device(const char *deviceCode,const char *nodeid)
{
	cJSON *msg;
	cJSON *arg;

	kk_map_dev_node_t *dev;

	if(deviceCode==NULL||nodeid==NULL){
		return -1;
	}

	arg = cJSON_CreateObject();
	cJSON_AddStringToObject(arg,"mac",deviceCode);
	msg = old_ccu_msg_build_json(nodeid,"DEL_DEVICE",NULL,arg);
	send_msg_to_module(msg);
	cJSON_Delete(msg);

	return 0;
}

int del_device_all_node(const char *deviceCode)
{
	#define MAX_CHANNEL_NUM 				32
	int list[MAX_CHANNEL_NUM] = {0};
	int i,num = 0;
	char *nodeid;
	
	//删除数据库
	kk_lan_db_node_delete(deviceCode);
	
	num = kk_lan_db_node_get_all(deviceCode,list);
	for(i=0;i<num;i++){
		nodeid = node_string(list[i]);
		del_device(deviceCode,nodeid);
		free(nodeid);
	}
	if(num){
		kk_handle_sync_info();
	}
}



void device_delete_sync(cJSON *payload)
{
	cJSON *params;
	cJSON *deviceCode;

	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	
	params = cJSON_GetObjectItem(payload,"params");
	
	deviceCode = cJSON_GetObjectItem(params, "deviceCode");
	del_device_all_node(deviceCode->valuestring);
	


}
void device_add_sync(cJSON *payload)
{

	//NEW_DEVICES
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	kk_handle_sync_info();
}








void device_new_device_sync(cJSON *payload)
{

	//NEW_DEVICES
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);
	kk_handle_sync_info();
}


void property_post_deal(const char *deviceCode,cJSON *payload)
{
	cJSON *params;

	kk_map_dev_node_t *dev;
	printf("[%s][%d]\n",__FUNCTION__,__LINE__);

	if(kk_map_dev_search_by_deviceCode(deviceCode, &dev)==0){
		
		params = cJSON_GetObjectItem(payload,"params");
		attr_report_conver(deviceCode,dev,params);
	}
}

void property_syn_deal(const char *deviceCode,cJSON *properties)
{
	cJSON *params;

	kk_map_dev_node_t *dev;

	if(kk_map_dev_search_by_deviceCode(deviceCode, &dev)==0){
		attr_report_conver(deviceCode,dev,properties);
	}
}














